==> phoenix-starter-multilingual-0.1.1.9-20241226.053240-1-sources.jar: com/elitesland/cbpl/multilingual/service/MultilingualServiceImpl.java <==
package com.elitesland.cbpl.multilingual.service;

import cn.hutool.core.collection.CollUtil;
import com.elitesland.cbpl.formgenerator.entity.EnhanceFormDO;
import com.elitesland.cbpl.formgenerator.repo.EnhanceFormRepoProc;
import com.elitesland.cbpl.formgenerator.vo.resp.HistoryFieldVO;
import com.elitesland.cbpl.multilingual.cache.MultilingualCache;
import com.elitesland.cbpl.multilingual.domain.MultilingualBatchVO;
import com.elitesland.cbpl.multilingual.domain.MultilingualFieldVO;
import com.elitesland.cbpl.multilingual.domain.MultilingualFlatVO;
import com.elitesland.cbpl.multilingual.domain.MultilingualVO;
import com.elitesland.cbpl.multilingual.repo.MultilingualRepoProc;
import com.elitesland.cbpl.tool.core.bean.BeanUtils;
import com.fasterxml.jackson.core.type.TypeReference;
import lombok.RequiredArgsConstructor;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import static cn.hutool.core.text.CharSequenceUtil.toUnderlineCase;
import static com.elitesland.cbpl.multilingual.common.constant.MultilingualConstant.DB_FIELD_ID;
import static com.elitesland.cbpl.multilingual.common.constant.MultilingualConstant.DB_FIELD_LANGUAGE;

/**
 * @author eric.hao
 * @since 2024/12/02
 */
@Slf4j
@Service
@RequiredArgsConstructor
public class MultilingualServiceImpl implements MultilingualService {

    private final MultilingualRepoProc multilingualRepoProc;
    private final EnhanceFormRepoProc enhanceFormRepoProc;

    @SneakyThrows
    @Override
    public MultilingualVO queryTranslate(String formCode, Long bizKey) {
        // 翻译数据
        var multilingualVO = new MultilingualVO();
        multilingualVO.setFormCode(formCode);
        multilingualVO.setBizKey(bizKey);

        // 1.表单配置信息
        EnhanceFormDO formDO = enhanceFormRepoProc.findByFormCode(formCode);
        if (formDO == null) {
            throw new IllegalArgumentException("表单(" + formCode + ")不存在或未启用");
        }

        // 2.需要翻译的字段
        Class<?> tableClass = Class.forName(formDO.getTableClass());
        List<HistoryFieldVO> fields = MultilingualCache.languageFields(tableClass);

        // 3.查询业务数据
        Map<String, Object> data = multilingualRepoProc.fetchOne(tableClass, bizKey, fields);

        // 4.1 如果没有翻译内容，直接返回
        Object translate = data.get(DB_FIELD_LANGUAGE);
        String translateStr = translate == null ? "{}" : translate.toString();
        // 4.2 转义翻译内容
        var langPackVO = BeanUtils.toMap(translateStr, new TypeReference<Map<String, Map<String, String>>>() {
        });

        List<MultilingualFieldVO> fieldList = new ArrayList<>();
        for (HistoryFieldVO field : fields) {
            MultilingualFieldVO fieldVO = new MultilingualFieldVO(); // TODO 默认设置语言列表
            fieldVO.setFieldCode(field.getFieldCode());
            fieldVO.setFieldName(field.getFieldName());
            fieldVO.setFieldValue(data.get(field.getFieldCode()) == null ? "" : data.get(field.getFieldCode()).toString());
            // set multilingual translate
            for (String langKey : langPackVO.keySet()) {
                Map<String, String> fieldValues = langPackVO.get(langKey);
                fieldVO.addTranslation(langKey, fieldValues.get(toUnderlineCase(field.getFieldCode())));
            }
            fieldList.add(fieldVO);
        }
        multilingualVO.setFields(fieldList);
        return multilingualVO;
    }

    @SneakyThrows
    @Transactional(rollbackFor = Exception.class)
    @Override
    public Boolean updateTranslate(MultilingualVO multilingualVO) {
        if (CollUtil.isEmpty(multilingualVO.getFields())) {
            logger.warn("[Multilingual] update multilingual fields is empty]");
            return false;
        }
        // 表单配置信息
        EnhanceFormDO formDO = enhanceFormRepoProc.findByFormCode(multilingualVO.getFormCode());
        if (formDO == null) {
            throw new IllegalArgumentException("表单(" + multilingualVO.getFormCode() + ")不存在或未启用");
        }
        return multilingualRepoProc.update(Class.forName(formDO.getTableClass()), multilingualVO) > 0;
    }

    @SneakyThrows
    @Override
    public List<MultilingualFlatVO> exportData(MultilingualBatchVO paramVO) {
        List<MultilingualFlatVO> result = new ArrayList<>();

        // 1.表单配置信息
        EnhanceFormDO formDO = enhanceFormRepoProc.findByFormCode(paramVO.getFormCode());
        if (formDO == null) {
            throw new IllegalArgumentException("表单(" + paramVO.getFormCode() + ")不存在或未启用");
        }

        // 2.需要翻译的字段
        Class<?> tableClass = Class.forName(formDO.getTableClass());
        List<HistoryFieldVO> fields = MultilingualCache.languageFields(tableClass);

        // 3.查询业务数据
        List<Map<String, Object>> dataList = multilingualRepoProc.fetch(tableClass, fields);


        for (Map<String, Object> data : dataList) {

            for (HistoryFieldVO field : fields) {
                // 翻译数据
                var exportVO = new MultilingualFlatVO();
                exportVO.setFormCode(paramVO.getFormCode());
                exportVO.setBizKey(String.valueOf(data.get(DB_FIELD_ID)));

                exportVO.setFieldCode(field.getFieldCode());
                exportVO.setFieldName(field.getFieldName());
                exportVO.setFieldValue(String.valueOf(data.get(field.getFieldCode())));

                // 4.1 如果没有翻译内容，直接返回
                String translate = String.valueOf(data.get(DB_FIELD_LANGUAGE));
                if (paramVO.getLanguage() != null && translate != null) {
                    // 4.2 转义翻译内容
                    var langPackVO = BeanUtils.toMap(translate, new TypeReference<Map<String, Map<String, String>>>() {
                    });
                    if (langPackVO != null) {
                        langPackVO.entrySet().stream().filter(entry -> entry.getKey().equals(paramVO.getLanguage())).forEach(entry -> {
                            exportVO.setTranslate(entry.getValue().get(toUnderlineCase(field.getFieldCode())));
                        });
                    }
                }

                result.add(exportVO);
            }

        }
        return result;
    }

    @SneakyThrows
    @Transactional(rollbackFor = Exception.class)
    @Override
    public List<String> importData(MultilingualBatchVO paramVO) {
        List<String> errorMessages = new ArrayList<>();

        // 1.表单配置信息
        EnhanceFormDO formDO = enhanceFormRepoProc.findByFormCode(paramVO.getFormCode());
        if (formDO == null) {
            throw new IllegalArgumentException("表单(" + paramVO.getFormCode() + ")不存在或未启用");
        }

        Map<String, List<MultilingualFlatVO>> collect = paramVO.getData().stream().collect(Collectors.groupingBy(MultilingualFlatVO::getBizKey));

        List<MultilingualVO> importData = new ArrayList<>();

        collect.forEach((bizKey, values) -> {
            MultilingualVO multilingualVO = new MultilingualVO();
            multilingualVO.setFormCode(paramVO.getFormCode());
            // TODO 检查数字类型
            multilingualVO.setBizKey(Long.valueOf(bizKey));

            List<MultilingualFieldVO> fieldList = new ArrayList<>();
            for (var field : values) {
                MultilingualFieldVO fieldVO = new MultilingualFieldVO(); // TODO 默认设置语言列表
                fieldVO.setFieldCode(field.getFieldCode());
                fieldVO.setFieldName(field.getFieldName());
                fieldVO.addTranslation(paramVO.getLanguage(), field.getTranslate());
                fieldList.add(fieldVO);
            }
            multilingualVO.setFields(fieldList);

            importData.add(multilingualVO);
        });
        Class<?> tableClass = Class.forName(formDO.getTableClass());
        importData.forEach(row -> multilingualRepoProc.update(tableClass, row));

        return errorMessages;
    }
}

==> phoenix-starter-multilingual-0.1.1.12-20241231.062626-1-sources.jar: com/elitesland/cbpl/multilingual/service/MultilingualServiceImpl.java <==
package com.elitesland.cbpl.multilingual.service;

import cn.hutool.core.collection.CollUtil;
import com.elitesland.cbpl.formgenerator.entity.EnhanceFormDO;
import com.elitesland.cbpl.formgenerator.repo.EnhanceFormRepoProc;
import com.elitesland.cbpl.formgenerator.vo.resp.HistoryFieldVO;
import com.elitesland.cbpl.multilingual.cache.MultilingualCache;
import com.elitesland.cbpl.multilingual.domain.MultilingualBatchVO;
import com.elitesland.cbpl.multilingual.domain.MultilingualFieldVO;
import com.elitesland.cbpl.multilingual.domain.MultilingualFlatVO;
import com.elitesland.cbpl.multilingual.domain.MultilingualVO;
import com.elitesland.cbpl.multilingual.repo.MultilingualRepoProc;
import com.elitesland.cbpl.tool.core.bean.BeanUtils;
import com.fasterxml.jackson.core.type.TypeReference;
import lombok.RequiredArgsConstructor;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import static cn.hutool.core.text.CharSequenceUtil.toUnderlineCase;
import static com.elitesland.cbpl.multilingual.common.constant.MultilingualConstant.DB_FIELD_ID;
import static com.elitesland.cbpl.multilingual.common.constant.MultilingualConstant.DB_FIELD_LANGUAGE;

/**
 * @author eric.hao
 * @since 2024/12/02
 */
@Slf4j
@Service
@RequiredArgsConstructor
public class MultilingualServiceImpl implements MultilingualService {

    private final MultilingualRepoProc multilingualRepoProc;
    private final EnhanceFormRepoProc enhanceFormRepoProc;

    @SneakyThrows
    @Override
    public MultilingualVO queryTranslate(String formCode, Long bizKey) {
        // 翻译数据
        var multilingualVO = new MultilingualVO();
        multilingualVO.setFormCode(formCode);
        multilingualVO.setBizKey(bizKey);

        // 1.表单配置信息
        EnhanceFormDO formDO = enhanceFormRepoProc.findByFormCode(formCode);
        if (formDO == null) {
            throw new IllegalArgumentException("表单(" + formCode + ")不存在或未启用");
        }

        // 2.需要翻译的字段
        Class<?> tableClass = Class.forName(formDO.getTableClass());
        List<HistoryFieldVO> fields = MultilingualCache.languageFields(tableClass);

        // 3.查询业务数据
        Map<String, Object> data = multilingualRepoProc.fetchOne(tableClass, bizKey, fields);

        // 4.1 如果没有翻译内容，直接返回
        Object translate = data.get(DB_FIELD_LANGUAGE);
        String translateStr = translate == null ? "{}" : translate.toString();
        // 4.2 转义翻译内容
        var langPackVO = BeanUtils.toMap(translateStr, new TypeReference<Map<String, Map<String, String>>>() {
        });

        List<MultilingualFieldVO> fieldList = new ArrayList<>();
        for (HistoryFieldVO field : fields) {
            MultilingualFieldVO fieldVO = new MultilingualFieldVO(); // TODO 默认设置语言列表
            fieldVO.setFieldCode(field.getFieldCode());
            fieldVO.setFieldName(field.getFieldName());
            fieldVO.setFieldValue(data.get(field.getFieldCode()) == null ? "" : data.get(field.getFieldCode()).toString());
            // set multilingual translate
            for (String langKey : langPackVO.keySet()) {
                Map<String, String> fieldValues = langPackVO.get(langKey);
                fieldVO.addTranslation(langKey, fieldValues.get(toUnderlineCase(field.getFieldCode())));
            }
            fieldList.add(fieldVO);
        }
        multilingualVO.setFields(fieldList);
        return multilingualVO;
    }

    @SneakyThrows
    @Transactional(rollbackFor = Exception.class)
    @Override
    public Boolean updateTranslate(MultilingualVO multilingualVO) {
        if (CollUtil.isEmpty(multilingualVO.getFields())) {
            logger.warn("[Multilingual] update multilingual fields is empty]");
            return false;
        }
        // 表单配置信息
        EnhanceFormDO formDO = enhanceFormRepoProc.findByFormCode(multilingualVO.getFormCode());
        if (formDO == null) {
            throw new IllegalArgumentException("表单(" + multilingualVO.getFormCode() + ")不存在或未启用");
        }
        return multilingualRepoProc.update(Class.forName(formDO.getTableClass()), multilingualVO) > 0;
    }

    @SneakyThrows
    @Override
    public List<MultilingualFlatVO> exportData(MultilingualBatchVO paramVO) {
        List<MultilingualFlatVO> result = new ArrayList<>();

        // 1.表单配置信息
        EnhanceFormDO formDO = enhanceFormRepoProc.findByFormCode(paramVO.getFormCode());
        if (formDO == null) {
            throw new IllegalArgumentException("表单(" + paramVO.getFormCode() + ")不存在或未启用");
        }

        // 2.需要翻译的字段
        Class<?> tableClass = Class.forName(formDO.getTableClass());
        List<HistoryFieldVO> fields = MultilingualCache.languageFields(tableClass);

        // 3.查询业务数据
        List<Map<String, Object>> dataList = multilingualRepoProc.fetch(tableClass, fields);


        for (Map<String, Object> data : dataList) {

            for (HistoryFieldVO field : fields) {
                // 翻译数据
                var exportVO = new MultilingualFlatVO();
                exportVO.setFormCode(paramVO.getFormCode());
                exportVO.setBizKey(String.valueOf(data.get(DB_FIELD_ID)));

                exportVO.setFieldCode(field.getFieldCode());
                exportVO.setFieldName(field.getFieldName());
                exportVO.setFieldValue(String.valueOf(data.get(field.getFieldCode())));

                // 4.1 如果没有翻译内容，直接返回
                String translate = String.valueOf(data.get(DB_FIELD_LANGUAGE));
                if (paramVO.getLanguage() != null && translate != null) {
                    // 4.2 转义翻译内容
                    var langPackVO = BeanUtils.toMap(translate, new TypeReference<Map<String, Map<String, String>>>() {
                    });
                    if (langPackVO != null) {
                        langPackVO.entrySet().stream().filter(entry -> entry.getKey().equals(paramVO.getLanguage())).forEach(entry -> {
                            exportVO.setTranslate(entry.getValue().get(toUnderlineCase(field.getFieldCode())));
                        });
                    }
                }

                result.add(exportVO);
            }

        }
        return result;
    }

    @SneakyThrows
    @Transactional(rollbackFor = Exception.class)
    @Override
    public List<String> importData(MultilingualBatchVO paramVO) {
        List<String> errorMessages = new ArrayList<>();
        // 1.表单配置信息
        EnhanceFormDO formDO = enhanceFormRepoProc.findByFormCode(paramVO.getFormCode());
        if (formDO == null) {
            throw new IllegalArgumentException("表单(" + paramVO.getFormCode() + ")不存在或未启用");
        }
        var dataList = paramVO.getData().stream().collect(Collectors.groupingBy(MultilingualFlatVO::getBizKey));
        var importData = new ArrayList<MultilingualVO>();
        dataList.forEach((bizKey, values) -> {
            try {
                MultilingualVO multilingualVO = new MultilingualVO();
                multilingualVO.setFormCode(paramVO.getFormCode());
                // TODO 检查数字类型
                multilingualVO.setBizKey(Long.valueOf(bizKey));
                List<MultilingualFieldVO> fieldList = new ArrayList<>();
                for (var field : values) {
                    MultilingualFieldVO fieldVO = new MultilingualFieldVO();
                    fieldVO.setFieldCode(field.getFieldCode());
                    fieldVO.setFieldName(field.getFieldName());
                    fieldVO.addTranslation(paramVO.getLanguage(), field.getTranslate());
                    fieldList.add(fieldVO);
                }
                multilingualVO.setFields(fieldList);
                importData.add(multilingualVO);
            } catch (Exception e) {
                errorMessages.add("bizKey(" + bizKey + ") translate error: " + e.getMessage());
            }
        });
        Class<?> tableClass = Class.forName(formDO.getTableClass());
        // TODO 持续观察，是否优化批量更新方式
        importData.forEach(row -> multilingualRepoProc.update(tableClass, row));
        return errorMessages;
    }
}
